@@ -33,35 +33,39 @@ from plone.memoize.view import memoize
 
 class ManageContextualPortlets(BrowserView):
     implements(IManageContextualPortletsView)
-    
+
     def __init__(self, context, request):
         super(ManageContextualPortlets, self).__init__(context, request)
         self.request.set('disable_border', True)
-        
+
     # IManagePortletsView implementation
-    
+
+    @property
+    def macros(self):
+        return self.index.macros
+
     @property
     def category(self):
         return CONTEXT_CATEGORY
-        
+
     @property
     def key(self):
         return '/'.join(self.context.getPhysicalPath())
-    
+
     def getAssignmentMappingUrl(self, manager):
         baseUrl = str(getMultiAdapter((self.context, self.request), name='absolute_url'))
         return '%s/++contextportlets++%s' % (baseUrl, manager.__name__)
-    
+
     def getAssignmentsForManager(self, manager):
         assignments = getMultiAdapter((self.context, manager), IPortletAssignmentMapping)
         return assignments.values()
-    
+
     # view @@manage-portlets
-    
+
     def has_legacy_portlets(self):
         left_slots = getattr(aq_base(self.context), 'left_slots', [])
         right_slots = getattr(aq_base(self.context), 'right_slots', [])
-        
+
         return (left_slots or right_slots)
 
     # view @@set-portlet-blacklist-status
@@ -69,9 +73,9 @@ class ManageContextualPortlets(BrowserView):
         portletManager = getUtility(IPortletManager, name=manager)
         assignable = getMultiAdapter((self.context, portletManager,), ILocalPortletAssignmentManager)
         assignments = getMultiAdapter((self.context, portletManager), IPortletAssignmentMapping)
-        
+
         IPortletPermissionChecker(assignments.__of__(aq_inner(self.context)))()
-        
+
         def int2status(status):
             if status == 0:
                 return None
@@ -79,34 +83,38 @@ class ManageContextualPortlets(BrowserView):
                 return True
             else:
                 return False
-        
+
         assignable.setBlacklistStatus(GROUP_CATEGORY, int2status(group_status))
         assignable.setBlacklistStatus(CONTENT_TYPE_CATEGORY, int2status(content_type_status))
         assignable.setBlacklistStatus(CONTEXT_CATEGORY, int2status(context_status))
-        
+
         baseUrl = str(getMultiAdapter((self.context, self.request), name='absolute_url'))
         self.request.response.redirect(baseUrl + '/@@manage-portlets')
         return ''
-    
+
     # view @@convert-legacy-portlets
-    
+
     def convert_legacy_portlets(self):
         utils.convert_legacy_portlets(self.context)
         self.context.request.response.redirect(self.context.absolute_url() + '/@@manage-portlets')
 
 class ManageDashboardPortlets(BrowserView):
     implements(IManageDashboardPortletsView)
-        
+
     # IManagePortletsView implementation
-    
+
+    @property
+    def macros(self):
+        return self.index.macros
+
     @property
     def category(self):
         return USER_CATEGORY
-        
+
     @property
     def key(self):
         return self._getUserId()
-    
+
     def getAssignmentMappingUrl(self, manager):
         baseUrl = str(getMultiAdapter((self.context, self.request), name='absolute_url'))
         userId = self._getUserId()
@@ -118,39 +126,45 @@ class ManageDashboardPortlets(BrowserView):
         category = column[USER_CATEGORY]
         mapping = category.get(userId, None)
         if mapping is None:
-            mapping = category[userId] = UserPortletAssignmentMapping()
+            mapping = category[userId] = UserPortletAssignmentMapping(manager=manager.__name__,
+                                                                      category=USER_CATEGORY,
+                                                                      name=userId)
         return mapping.values()
-    
+
     def _getUserId(self):
         membership = getToolByName(aq_inner(self.context), 'portal_membership', None)
         if membership.isAnonymousUser():
             raise Unauthorized, "Cannot get portlet assignments for anonymous through this view"
-        
+
         member = membership.getAuthenticatedMember()
         memberId = member.getId()
-        
+
         if memberId is None:
-            raise KeyError, "Cannot find user id of current user" 
-        
+            raise KeyError, "Cannot find user id of current user"
+
         return memberId
 
 class ManageGroupPortlets(BrowserView):
     implements(IManageGroupPortletsView)
-        
+
     # IManagePortletsView implementation
-    
+
+    @property
+    def macros(self):
+        return self.index.macros
+
     @property
     def category(self):
         return GROUP_CATEGORY
-        
+
     @property
     def key(self):
         return self.request['key']
-    
+
     def __init__(self, context, request):
         super(ManageGroupPortlets, self).__init__(context, request)
         self.request.set('disable_border', True)
-    
+
     def getAssignmentMappingUrl(self, manager):
         baseUrl = str(getMultiAdapter((self.context, self.request), name='absolute_url'))
         key = self.request['key']
@@ -162,31 +176,37 @@ class ManageGroupPortlets(BrowserView):
         category = column[GROUP_CATEGORY]
         mapping = category.get(key, None)
         if mapping is None:
-            mapping = category[key] = PortletAssignmentMapping()
+            mapping = category[key] = PortletAssignmentMapping(manager=manager.__name__,
+                                                               category=GROUP_CATEGORY,
+                                                               name=key)
         return mapping.values()
-    
+
     # View attributes
-    
+
     def group(self):
         return self.request['key']
 
 class ManageContentTypePortlets(BrowserView):
     implements(IManageContentTypePortletsView)
-    
+
     def __init__(self, context, request):
         super(ManageContentTypePortlets, self).__init__(context, request)
         self.request.set('disable_border', True)
-        
+
     # IManagePortletsView implementation
-    
+
+    @property
+    def macros(self):
+        return self.index.macros
+
     @property
     def category(self):
         return CONTENT_TYPE_CATEGORY
-        
+
     @property
     def key(self):
         return self.request['key']
-    
+
     def getAssignmentMappingUrl(self, manager):
         baseUrl = str(getMultiAdapter((self.context, self.request), name='absolute_url'))
         pt = self.request['key']
@@ -198,22 +218,89 @@ class ManageContentTypePortlets(BrowserView):
         category = column[CONTENT_TYPE_CATEGORY]
         mapping = category.get(pt, None)
         if mapping is None:
-            mapping = category[pt] = PortletAssignmentMapping()
+            mapping = category[pt] = PortletAssignmentMapping(manager=manager.__name__,
+                                                              category=CONTENT_TYPE_CATEGORY,
+                                                              name=pt)
         return mapping.values()
-    
+
     # View attributes
-    
+
     def portal_type(self):
         return self.fti().Title()
-                
+
     def portal_type_icon(self):
         ploneview = getMultiAdapter((self.context, self.request), name=u"plone")
         return ploneview.getIcon(self.fti())
-           
+
     @memoize
     def fti(self):
         portal_types = getToolByName(aq_inner(self.context), 'portal_types')
         portal_type = self.request['key']
         for fti in portal_types.listTypeInfo():
             if fti.getId() == portal_type:
-                return fti
\ No newline at end of file
+                return fti
+
+class ManagePortletsViewlet(BrowserView):
+    """A general base class for viewlets that want to be rendered on the
+    manage portlets view. This makes it possible to have a viewlet that
+    renders a portlet manager, and still have the generic edit manager
+    renderer work (it doesn't work otherwise, because the edit manager
+    renderer is registered on IManagePortletsView, but inside a viewlet,
+    the __parent__ is the viewlet, not the ultimate parent).
+    """
+    implements(IManagePortletsView)
+
+    def __init__(self, context, request, view, manager):
+        super(ManagePortletsViewlet, self).__init__(context, request)
+        self.__parent__ = view
+        self.context = context
+        self.request = request
+        self.view = view
+        self.manager = manager
+
+    @property
+    def macros(self):
+        return self.index.macros
+
+    @property
+    def category(self):
+        return self.ultimate_parent().category
+
+    @property
+    def key(self):
+        return self.ultimate_parent().key
+
+    def getAssignmentMappingUrl(self, manager):
+        return self.ultimate_parent().getAssignmentMappingUrl(manager)
+
+    def getAssignmentsForManager(self, manager):
+        return self.ultimate_parent().getAssignmentsForManager(manager)
+
+    @memoize
+    def ultimate_parent(self):
+        parent = self.__parent__
+        while parent is not None and hasattr(parent, '__parent__') and parent.__parent__ is not None:
+            parent = parent.__parent__
+        return parent
+
+    # Subclasses need to implement update() and render() - or
+    # we can use ZCML with the template attribute (hence we don't
+    # put these here)
+
+class ManageContextualPortletsViewlet(ManagePortletsViewlet):
+    """A viewlet base class for viewlets that need to render on the
+    manage contextual portlets screen.
+    """
+    implements(IManageContextualPortletsView)
+
+class ManageGroupPortletsViewlet(ManagePortletsViewlet):
+    """A viewlet base class for viewlets that need to render on the
+    manage group portlets screen.
+    """
+    implements(IManageGroupPortletsView)
+
+class ManageContentTypePortletsViewlet(ManagePortletsViewlet):
+    """A viewlet base class for viewlets that need to render on the
+    manage content type portlets screen.
+    """
+    implements(IManageContentTypePortletsView)
